package sun.rmi.server;

import java.lang.ref.Reference;
import java.lang.ref.SoftReference;
import java.util.WeakHashMap;
import java.util.Map;

public abstract class WeakClassHashMap<V>
{
    private Map<Class<?>, WeakClassHashMap.ValueCell<V>> internalMap;
    
    protected WeakClassHashMap() {
        this.internalMap = new WeakHashMap();
    }
    
    public V get(Class<?> clazz) {
        WeakClassHashMap.ValueCell<Object> valueCell;
        synchronized (this.internalMap) {
            valueCell = (WeakClassHashMap.ValueCell)this.internalMap.get(clazz);
            if (valueCell == null) {
                valueCell = new WeakClassHashMap.ValueCell();
                this.internalMap.put(clazz, valueCell);
            }
        }
        synchronized (valueCell) {
            Object referent = null;
            if (valueCell.ref != null) {
                referent = valueCell.ref.get();
            }
            if (referent == null) {
                referent = this.computeValue(clazz);
                valueCell.ref = new SoftReference(referent);
                return referent;
            }
            return referent;
        }
    }
    
    protected abstract V computeValue(Class<?> p0);
    
    private static class ValueCell<T>
    {
        Reference<T> ref;
        
        ValueCell() {
            this.ref = null;
        }
    }
}
